-- $Id: bwaa-analysis_types.ads 61 2009-04-05 18:27:40Z georg $
---------------------------------------------------------------------------
--                                                                       --
--                   BWAA - Busy Wait Analyser for Ada                   --
--      BWAA is a program that finds busy waiting in Ada programs.       --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--                 Copyright (c) 2009 Georg Kienesberger                 --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of BWAA.                                            --
--                                                                       --
-- BWAA is free software; you can redistribute it and/or modify          --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- BWAA is distributed in the hope that it will be useful,               --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Ada.Containers.Hashed_Sets;

with Ast2Cfg;

with Bwaa.Util; use Bwaa.Util;

package Bwaa.Analysis_Types is

   type Variable is private;
   procedure Init(V: in out Variable; Name: in Unbounded_String);
   procedure Init(V: in out Variable; Name: in String);
   function "=" (Left: in Variable; Right: in Variable) return Boolean;
   function Get_Name(V: in Variable) return Unbounded_String;
   function Get_Name(V: in Variable) return String;
   procedure Set_Busy_Var(V: in out Variable);
   function Is_Busy_Var(V: in Variable) return Boolean;
   procedure Set_Direct_Candidate_Name(V: in out Variable;
                                     Direct_Candidate_Name: in Unbounded_String);
   procedure Set_Direct_Candidate_Name(V: in out Variable;
                                     Direct_Candidate_Name: in String);
   function Get_Direct_Candidate_Name(V: in Variable) return Unbounded_String;
   function Get_Direct_Candidate_Name(V: in Variable) return String;
   function Hash (V: in Variable) return Ada.Containers.Hash_Type;

private

   type Variable is
      record
         Name: Unbounded_String := Null_Unbounded_String;
         Busy_Var: Boolean := False;
         Direct_Candidate_Name: Unbounded_String := Null_Unbounded_String;
      end record;

end Bwaa.Analysis_Types;
