-- $Id: bwaa-visibility-visibility_info.ads 96 2009-06-05 14:51:16Z georg $
---------------------------------------------------------------------------
--                                                                       --
--                   BWAA - Busy Wait Analyser for Ada                   --
--      BWAA is a program that finds busy waiting in Ada programs.       --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--                 Copyright (c) 2009 Georg Kienesberger                 --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of BWAA.                                            --
--                                                                       --
-- BWAA is free software; you can redistribute it and/or modify          --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- BWAA is distributed in the hope that it will be useful,               --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ast2Cfg.Flow_Types; use Ast2Cfg.Flow_Types;

with Bwaa.Util;

package Bwaa.Visibility.Visibility_Info is

   package Visible_Set is new Ada.Containers.Hashed_Sets (Element_Type => Declaration,
                                                          Hash => Hash,
                                                          Equivalent_Elements => "=");

   package Visibility_Map is new Ada.Containers.Hashed_Maps (Key_Type => Unbounded_String,
                                                             Element_Type => Visible_Set.Set,
                                                             Hash => Ada.Strings.Unbounded.Hash,
                                                             Equivalent_Keys => Ada.Strings.Unbounded."=",
                                                             "=" => Visible_Set."=");

   type Info is limited
      record
         Specs: Visibility_Map.Map := Visibility_Map.Empty_Map;
         Bodies: Visibility_Map.Map := Visibility_Map.Empty_Map;
         Pkg_Specs: Visibility_Map.Map := Visibility_Map.Empty_Map;
         Pkg_Bodies: Visibility_Map.Map := Visibility_Map.Empty_Map;
      end record;

   procedure Get_Full_Visible_Name (Full_Old_Var_Name: in out Unbounded_String;
                                    Decl_Id: in Ast2Cfg.Declaration_Id.Object;
                                    Flow: in Ast2Cfg.Flow_Types.Flow_Object'Class;
                                    Specs: in Bwaa.Util.Specification_Map.Map;
                                    Vm_Specs: in Visibility_Map.Map;
                                    Vm_Bodies: in Visibility_Map.Map;
                                    Last_Try: in Boolean := False);

private

   procedure Try_At_Pred (Full_Old_Var_Name: in out Unbounded_String;
                          Flow: in Ast2Cfg.Flow_Types.Flow_Object'Class;
                          Specs: in Bwaa.Util.Specification_Map.Map;
                          Vm_Specs: in Visibility_Map.Map;
                          Vm_Bodies: in Visibility_Map.Map);

end Bwaa.Visibility.Visibility_Info;
