-- $Id: bwaa-visibility.ads 95 2009-06-05 14:49:48Z georg $
---------------------------------------------------------------------------
--                                                                       --
--                   BWAA - Busy Wait Analyser for Ada                   --
--      BWAA is a program that finds busy waiting in Ada programs.       --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--                 Copyright (c) 2009 Georg Kienesberger                 --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of BWAA.                                            --
--                                                                       --
-- BWAA is free software; you can redistribute it and/or modify          --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- BWAA is distributed in the hope that it will be useful,               --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Ada.Strings.Unbounded.Hash;
with Ada.Containers.Hashed_Maps;
with Ada.Containers.Hashed_Sets;

with Ast2Cfg.Declaration_Id;

with Bwaa.Util;

package Bwaa.Visibility is

   type Declaration is private;

   procedure Init (Decl: in out Declaration;
                   Name: in Unbounded_String;
                   Prefix: in Unbounded_String;
                   Id: in Ast2Cfg.Declaration_Id.Object);

   function "=" (Left: in Declaration; Right: in Declaration) return Boolean;

   function Before (Left: in Declaration; Right: in Declaration) return Boolean;
   function After (Left: in Declaration; Right: in Declaration) return Boolean;
   function "<" (Left: in Declaration; Right: in Declaration) return Boolean renames Before;
   function ">" (Left: in Declaration; Right: in Declaration) return Boolean renames After;

   function Before (Left: in Declaration; Right: in Ast2Cfg.Declaration_Id.Object) return Boolean;
   function Before (Left: in Ast2Cfg.Declaration_Id.Object; Right: in Declaration) return Boolean;
   function After (Left: in Declaration; Right: in Ast2Cfg.Declaration_Id.Object) return Boolean;
   function After (Left: in Ast2Cfg.Declaration_Id.Object; Right: in Declaration) return Boolean;
   function "<" (Left: in Declaration; Right: in Ast2Cfg.Declaration_Id.Object) return Boolean renames Before;
   function "<" (Left: in Ast2Cfg.Declaration_Id.Object; Right: in Declaration) return Boolean renames Before;
   function ">" (Left: in Declaration; Right: in Ast2Cfg.Declaration_Id.Object) return Boolean renames After;
   function ">" (Left: in Ast2Cfg.Declaration_Id.Object; Right: in Declaration) return Boolean renames After;

   function Get_Name (Decl: in Declaration) return Unbounded_String;
   function Get_Prefix (Decl: in Declaration) return Unbounded_String;
   function Get_Full_Name (Decl: in Declaration) return Unbounded_String;

   function Get_Id (Decl: in Declaration) return Ast2Cfg.Declaration_Id.Object;

   function Hash (Decl: in Declaration) return Ada.Containers.Hash_Type;

   Visibility_Exception: exception;

private

   type Declaration is
      record
         Name: Unbounded_String := Null_Unbounded_String;
         Prefix: Unbounded_String := Null_Unbounded_String;
         Id: Ast2Cfg.Declaration_Id.Object := Ast2Cfg.Declaration_Id.Null_Declaration_Id;
      end record;

end Bwaa.Visibility;
