-- $Id: run_bwaa.adb 109 2009-06-14 17:14:39Z georg $
---------------------------------------------------------------------------
--                                                                       --
--                   BWAA - Busy Wait Analyser for Ada                   --
--      BWAA is a program that finds busy waiting in Ada programs.       --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--                 Copyright (c) 2009 Georg Kienesberger                 --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of BWAA.                                            --
--                                                                       --
-- BWAA is free software; you can redistribute it and/or modify          --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- BWAA is distributed in the hope that it will be useful,               --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------
with Ada.Text_IO; use Ada.Text_IO;
with Ada.Command_Line; use Ada.Command_Line;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

with Ast2Cfg;

with Bwaa.Output;
with Bwaa.Main;
with Bwaa.Options;
use Bwaa;

procedure Run_Bwaa is

   procedure Usage is
   begin
      Put_Line(Standard_Error, "BWAA version " & Options.Version & ", Copyright (c) 2009 Georg Kienesberger");
      Put_Line(Standard_Error, "BWAA comes with ABSOLUTELY NO WARRANTY; for details type `" & Command_Name & " -l'.");
      Put_Line(Standard_Error, "This is free software, and you are welcome to redistribute it under certain");
      Put_Line(Standard_Error, "conditions; type `" & Command_Name & " -l' for details.");
      Put_Line(Standard_Error, "");
      Put_Line(Standard_Error, "Using Ast2Cfg version: " & Ast2Cfg.Version & "");
      Put_Line(Standard_Error, "");
      Put_Line(Standard_Error, "Usage: " & Command_Name & " [-v|-vv|-vvv|-vvvv] [-i] [-t] [-f FILE1 FILE2 .. FILEN]");
      Put_Line(Standard_Error, "Without options all adt files in the current directory are processed.");
      Put_Line(Standard_Error, "");
      Put_Line(Standard_Error, "Options:");
      Put_Line(Standard_Error, " -f     only consider adt files FILE1 to FILEN");
      Put_Line(Standard_Error, " -i     also consider indirect candidates");
      Put_Line(Standard_Error, " -t     only report busy wait if every variable in a termination");
      Put_Line(Standard_Error, "        statement is affected (light scan)");
      Put_Line(Standard_Error, " -v     be verbose (show pretty printed output)");
      Put_Line(Standard_Error, " -vv    be more verbose (show detailed output)");
      Put_Line(Standard_Error, " -vvv   be very verbose (show debug output)");
      Put_Line(Standard_Error, " -vvvv  be very very verbose (show all output, not recommended)");
   end Usage;

   procedure Show_GPL is
   begin
      Put_Line("                    GNU GENERAL PUBLIC LICENSE");
      Put_Line("                       Version 3, 29 June 2007");
      Put_Line("");
      Put_Line(" Copyright (C) 2007 Free Software Foundation, Inc. <http://fsf.org/>");
      Put_Line(" Everyone is permitted to copy and distribute verbatim copies");
      Put_Line(" of this license document, but changing it is not allowed.");
      Put_Line("");
      Put_Line("                            Preamble");
      Put_Line("");
      Put_Line("  The GNU General Public License is a free, copyleft license for");
      Put_Line("software and other kinds of works.");
      Put_Line("");
      Put_Line("  The licenses for most software and other practical works are designed");
      Put_Line("to take away your freedom to share and change the works.  By contrast,");
      Put_Line("the GNU General Public License is intended to guarantee your freedom to");
      Put_Line("share and change all versions of a program--to make sure it remains free");
      Put_Line("software for all its users.  We, the Free Software Foundation, use the");
      Put_Line("GNU General Public License for most of our software; it applies also to");
      Put_Line("any other work released this way by its authors.  You can apply it to");
      Put_Line("your programs, too.");
      Put_Line("");
      Put_Line("  When we speak of free software, we are referring to freedom, not");
      Put_Line("price.  Our General Public Licenses are designed to make sure that you");
      Put_Line("have the freedom to distribute copies of free software (and charge for");
      Put_Line("them if you wish), that you receive source code or can get it if you");
      Put_Line("want it, that you can change the software or use pieces of it in new");
      Put_Line("free programs, and that you know you can do these things.");
      Put_Line("");
      Put_Line("  To protect your rights, we need to prevent others from denying you");
      Put_Line("these rights or asking you to surrender the rights.  Therefore, you have");
      Put_Line("certain responsibilities if you distribute copies of the software, or if");
      Put_Line("you modify it: responsibilities to respect the freedom of others.");
      Put_Line("");
      Put_Line("  For example, if you distribute copies of such a program, whether");
      Put_Line("gratis or for a fee, you must pass on to the recipients the same");
      Put_Line("freedoms that you received.  You must make sure that they, too, receive");
      Put_Line("or can get the source code.  And you must show them these terms so they");
      Put_Line("know their rights.");
      Put_Line("");
      Put_Line("  Developers that use the GNU GPL protect your rights with two steps:");
      Put_Line("(1) assert copyright on the software, and (2) offer you this License");
      Put_Line("giving you legal permission to copy, distribute and/or modify it.");
      Put_Line("");
      Put_Line("  For the developers' and authors' protection, the GPL clearly explains");
      Put_Line("that there is no warranty for this free software.  For both users' and");
      Put_Line("authors' sake, the GPL requires that modified versions be marked as");
      Put_Line("changed, so that their problems will not be attributed erroneously to");
      Put_Line("authors of previous versions.");
      Put_Line("");
      Put_Line("  Some devices are designed to deny users access to install or run");
      Put_Line("modified versions of the software inside them, although the manufacturer");
      Put_Line("can do so.  This is fundamentally incompatible with the aim of");
      Put_Line("protecting users' freedom to change the software.  The systematic");
      Put_Line("pattern of such abuse occurs in the area of products for individuals to");
      Put_Line("use, which is precisely where it is most unacceptable.  Therefore, we");
      Put_Line("have designed this version of the GPL to prohibit the practice for those");
      Put_Line("products.  If such problems arise substantially in other domains, we");
      Put_Line("stand ready to extend this provision to those domains in future versions");
      Put_Line("of the GPL, as needed to protect the freedom of users.");
      Put_Line("");
      Put_Line("  Finally, every program is threatened constantly by software patents.");
      Put_Line("States should not allow patents to restrict development and use of");
      Put_Line("software on general-purpose computers, but in those that do, we wish to");
      Put_Line("avoid the special danger that patents applied to a free program could");
      Put_Line("make it effectively proprietary.  To prevent this, the GPL assures that");
      Put_Line("patents cannot be used to render the program non-free.");
      Put_Line("");
      Put_Line("  The precise terms and conditions for copying, distribution and");
      Put_Line("modification follow.");
      Put_Line("");
      Put_Line("                       TERMS AND CONDITIONS");
      Put_Line("");
      Put_Line("  0. Definitions.");
      Put_Line("");
      Put_Line("  ""This License"" refers to version 3 of the GNU General Public License.");
      Put_Line("");
      Put_Line("  ""Copyright"" also means copyright-like laws that apply to other kinds of");
      Put_Line("works, such as semiconductor masks.");
      Put_Line("");
      Put_Line("  ""The Program"" refers to any copyrightable work licensed under this");
      Put_Line("License.  Each licensee is addressed as ""you"".  ""Licensees"" and");
      Put_Line("""recipients"" may be individuals or organizations.");
      Put_Line("");
      Put_Line("  To ""modify"" a work means to copy from or adapt all or part of the work");
      Put_Line("in a fashion requiring copyright permission, other than the making of an");
      Put_Line("exact copy.  The resulting work is called a ""modified version"" of the");
      Put_Line("earlier work or a work ""based on"" the earlier work.");
      Put_Line("");
      Put_Line("  A ""covered work"" means either the unmodified Program or a work based");
      Put_Line("on the Program.");
      Put_Line("");
      Put_Line("  To ""propagate"" a work means to do anything with it that, without");
      Put_Line("permission, would make you directly or secondarily liable for");
      Put_Line("infringement under applicable copyright law, except executing it on a");
      Put_Line("computer or modifying a private copy.  Propagation includes copying,");
      Put_Line("distribution (with or without modification), making available to the");
      Put_Line("public, and in some countries other activities as well.");
      Put_Line("");
      Put_Line("  To ""convey"" a work means any kind of propagation that enables other");
      Put_Line("parties to make or receive copies.  Mere interaction with a user through");
      Put_Line("a computer network, with no transfer of a copy, is not conveying.");
      Put_Line("");
      Put_Line("  An interactive user interface displays ""Appropriate Legal Notices""");
      Put_Line("to the extent that it includes a convenient and prominently visible");
      Put_Line("feature that (1) displays an appropriate copyright notice, and (2)");
      Put_Line("tells the user that there is no warranty for the work (except to the");
      Put_Line("extent that warranties are provided), that licensees may convey the");
      Put_Line("work under this License, and how to view a copy of this License.  If");
      Put_Line("the interface presents a list of user commands or options, such as a");
      Put_Line("menu, a prominent item in the list meets this criterion.");
      Put_Line("");
      Put_Line("  1. Source Code.");
      Put_Line("");
      Put_Line("  The ""source code"" for a work means the preferred form of the work");
      Put_Line("for making modifications to it.  ""Object code"" means any non-source");
      Put_Line("form of a work.");
      Put_Line("");
      Put_Line("  A ""Standard Interface"" means an interface that either is an official");
      Put_Line("standard defined by a recognized standards body, or, in the case of");
      Put_Line("interfaces specified for a particular programming language, one that");
      Put_Line("is widely used among developers working in that language.");
      Put_Line("");
      Put_Line("  The ""System Libraries"" of an executable work include anything, other");
      Put_Line("than the work as a whole, that (a) is included in the normal form of");
      Put_Line("packaging a Major Component, but which is not part of that Major");
      Put_Line("Component, and (b) serves only to enable use of the work with that");
      Put_Line("Major Component, or to implement a Standard Interface for which an");
      Put_Line("implementation is available to the public in source code form.  A");
      Put_Line("""Major Component"", in this context, means a major essential component");
      Put_Line("(kernel, window system, and so on) of the specific operating system");
      Put_Line("(if any) on which the executable work runs, or a compiler used to");
      Put_Line("produce the work, or an object code interpreter used to run it.");
      Put_Line("");
      Put_Line("  The ""Corresponding Source"" for a work in object code form means all");
      Put_Line("the source code needed to generate, install, and (for an executable");
      Put_Line("work) run the object code and to modify the work, including scripts to");
      Put_Line("control those activities.  However, it does not include the work's");
      Put_Line("System Libraries, or general-purpose tools or generally available free");
      Put_Line("programs which are used unmodified in performing those activities but");
      Put_Line("which are not part of the work.  For example, Corresponding Source");
      Put_Line("includes interface definition files associated with source files for");
      Put_Line("the work, and the source code for shared libraries and dynamically");
      Put_Line("linked subprograms that the work is specifically designed to require,");
      Put_Line("such as by intimate data communication or control flow between those");
      Put_Line("subprograms and other parts of the work.");
      Put_Line("");
      Put_Line("  The Corresponding Source need not include anything that users");
      Put_Line("can regenerate automatically from other parts of the Corresponding");
      Put_Line("Source.");
      Put_Line("");
      Put_Line("  The Corresponding Source for a work in source code form is that");
      Put_Line("same work.");
      Put_Line("");
      Put_Line("  2. Basic Permissions.");
      Put_Line("");
      Put_Line("  All rights granted under this License are granted for the term of");
      Put_Line("copyright on the Program, and are irrevocable provided the stated");
      Put_Line("conditions are met.  This License explicitly affirms your unlimited");
      Put_Line("permission to run the unmodified Program.  The output from running a");
      Put_Line("covered work is covered by this License only if the output, given its");
      Put_Line("content, constitutes a covered work.  This License acknowledges your");
      Put_Line("rights of fair use or other equivalent, as provided by copyright law.");
      Put_Line("");
      Put_Line("  You may make, run and propagate covered works that you do not");
      Put_Line("convey, without conditions so long as your license otherwise remains");
      Put_Line("in force.  You may convey covered works to others for the sole purpose");
      Put_Line("of having them make modifications exclusively for you, or provide you");
      Put_Line("with facilities for running those works, provided that you comply with");
      Put_Line("the terms of this License in conveying all material for which you do");
      Put_Line("not control copyright.  Those thus making or running the covered works");
      Put_Line("for you must do so exclusively on your behalf, under your direction");
      Put_Line("and control, on terms that prohibit them from making any copies of");
      Put_Line("your copyrighted material outside their relationship with you.");
      Put_Line("");
      Put_Line("  Conveying under any other circumstances is permitted solely under");
      Put_Line("the conditions stated below.  Sublicensing is not allowed; section 10");
      Put_Line("makes it unnecessary.");
      Put_Line("");
      Put_Line("  3. Protecting Users' Legal Rights From Anti-Circumvention Law.");
      Put_Line("");
      Put_Line("  No covered work shall be deemed part of an effective technological");
      Put_Line("measure under any applicable law fulfilling obligations under article");
      Put_Line("11 of the WIPO copyright treaty adopted on 20 December 1996, or");
      Put_Line("similar laws prohibiting or restricting circumvention of such");
      Put_Line("measures.");
      Put_Line("");
      Put_Line("  When you convey a covered work, you waive any legal power to forbid");
      Put_Line("circumvention of technological measures to the extent such circumvention");
      Put_Line("is effected by exercising rights under this License with respect to");
      Put_Line("the covered work, and you disclaim any intention to limit operation or");
      Put_Line("modification of the work as a means of enforcing, against the work's");
      Put_Line("users, your or third parties' legal rights to forbid circumvention of");
      Put_Line("technological measures.");
      Put_Line("");
      Put_Line("  4. Conveying Verbatim Copies.");
      Put_Line("");
      Put_Line("  You may convey verbatim copies of the Program's source code as you");
      Put_Line("receive it, in any medium, provided that you conspicuously and");
      Put_Line("appropriately publish on each copy an appropriate copyright notice;");
      Put_Line("keep intact all notices stating that this License and any");
      Put_Line("non-permissive terms added in accord with section 7 apply to the code;");
      Put_Line("keep intact all notices of the absence of any warranty; and give all");
      Put_Line("recipients a copy of this License along with the Program.");
      Put_Line("");
      Put_Line("  You may charge any price or no price for each copy that you convey,");
      Put_Line("and you may offer support or warranty protection for a fee.");
      Put_Line("");
      Put_Line("  5. Conveying Modified Source Versions.");
      Put_Line("");
      Put_Line("  You may convey a work based on the Program, or the modifications to");
      Put_Line("produce it from the Program, in the form of source code under the");
      Put_Line("terms of section 4, provided that you also meet all of these conditions:");
      Put_Line("");
      Put_Line("    a) The work must carry prominent notices stating that you modified");
      Put_Line("    it, and giving a relevant date.");
      Put_Line("");
      Put_Line("    b) The work must carry prominent notices stating that it is");
      Put_Line("    released under this License and any conditions added under section");
      Put_Line("    7.  This requirement modifies the requirement in section 4 to");
      Put_Line("    ""keep intact all notices"".");
      Put_Line("");
      Put_Line("    c) You must license the entire work, as a whole, under this");
      Put_Line("    License to anyone who comes into possession of a copy.  This");
      Put_Line("    License will therefore apply, along with any applicable section 7");
      Put_Line("    additional terms, to the whole of the work, and all its parts,");
      Put_Line("    regardless of how they are packaged.  This License gives no");
      Put_Line("    permission to license the work in any other way, but it does not");
      Put_Line("    invalidate such permission if you have separately received it.");
      Put_Line("");
      Put_Line("    d) If the work has interactive user interfaces, each must display");
      Put_Line("    Appropriate Legal Notices; however, if the Program has interactive");
      Put_Line("    interfaces that do not display Appropriate Legal Notices, your");
      Put_Line("    work need not make them do so.");
      Put_Line("");
      Put_Line("  A compilation of a covered work with other separate and independent");
      Put_Line("works, which are not by their nature extensions of the covered work,");
      Put_Line("and which are not combined with it such as to form a larger program,");
      Put_Line("in or on a volume of a storage or distribution medium, is called an");
      Put_Line("""aggregate"" if the compilation and its resulting copyright are not");
      Put_Line("used to limit the access or legal rights of the compilation's users");
      Put_Line("beyond what the individual works permit.  Inclusion of a covered work");
      Put_Line("in an aggregate does not cause this License to apply to the other");
      Put_Line("parts of the aggregate.");
      Put_Line("");
      Put_Line("  6. Conveying Non-Source Forms.");
      Put_Line("");
      Put_Line("  You may convey a covered work in object code form under the terms");
      Put_Line("of sections 4 and 5, provided that you also convey the");
      Put_Line("machine-readable Corresponding Source under the terms of this License,");
      Put_Line("in one of these ways:");
      Put_Line("");
      Put_Line("    a) Convey the object code in, or embodied in, a physical product");
      Put_Line("    (including a physical distribution medium), accompanied by the");
      Put_Line("    Corresponding Source fixed on a durable physical medium");
      Put_Line("    customarily used for software interchange.");
      Put_Line("");
      Put_Line("    b) Convey the object code in, or embodied in, a physical product");
      Put_Line("    (including a physical distribution medium), accompanied by a");
      Put_Line("    written offer, valid for at least three years and valid for as");
      Put_Line("    long as you offer spare parts or customer support for that product");
      Put_Line("    model, to give anyone who possesses the object code either (1) a");
      Put_Line("    copy of the Corresponding Source for all the software in the");
      Put_Line("    product that is covered by this License, on a durable physical");
      Put_Line("    medium customarily used for software interchange, for a price no");
      Put_Line("    more than your reasonable cost of physically performing this");
      Put_Line("    conveying of source, or (2) access to copy the");
      Put_Line("    Corresponding Source from a network server at no charge.");
      Put_Line("");
      Put_Line("    c) Convey individual copies of the object code with a copy of the");
      Put_Line("    written offer to provide the Corresponding Source.  This");
      Put_Line("    alternative is allowed only occasionally and noncommercially, and");
      Put_Line("    only if you received the object code with such an offer, in accord");
      Put_Line("    with subsection 6b.");
      Put_Line("");
      Put_Line("    d) Convey the object code by offering access from a designated");
      Put_Line("    place (gratis or for a charge), and offer equivalent access to the");
      Put_Line("    Corresponding Source in the same way through the same place at no");
      Put_Line("    further charge.  You need not require recipients to copy the");
      Put_Line("    Corresponding Source along with the object code.  If the place to");
      Put_Line("    copy the object code is a network server, the Corresponding Source");
      Put_Line("    may be on a different server (operated by you or a third party)");
      Put_Line("    that supports equivalent copying facilities, provided you maintain");
      Put_Line("    clear directions next to the object code saying where to find the");
      Put_Line("    Corresponding Source.  Regardless of what server hosts the");
      Put_Line("    Corresponding Source, you remain obligated to ensure that it is");
      Put_Line("    available for as long as needed to satisfy these requirements.");
      Put_Line("");
      Put_Line("    e) Convey the object code using peer-to-peer transmission, provided");
      Put_Line("    you inform other peers where the object code and Corresponding");
      Put_Line("    Source of the work are being offered to the general public at no");
      Put_Line("    charge under subsection 6d.");
      Put_Line("");
      Put_Line("  A separable portion of the object code, whose source code is excluded");
      Put_Line("from the Corresponding Source as a System Library, need not be");
      Put_Line("included in conveying the object code work.");
      Put_Line("");
      Put_Line("  A ""User Product"" is either (1) a ""consumer product"", which means any");
      Put_Line("tangible personal property which is normally used for personal, family,");
      Put_Line("or household purposes, or (2) anything designed or sold for incorporation");
      Put_Line("into a dwelling.  In determining whether a product is a consumer product,");
      Put_Line("doubtful cases shall be resolved in favor of coverage.  For a particular");
      Put_Line("product received by a particular user, ""normally used"" refers to a");
      Put_Line("typical or common use of that class of product, regardless of the status");
      Put_Line("of the particular user or of the way in which the particular user");
      Put_Line("actually uses, or expects or is expected to use, the product.  A product");
      Put_Line("is a consumer product regardless of whether the product has substantial");
      Put_Line("commercial, industrial or non-consumer uses, unless such uses represent");
      Put_Line("the only significant mode of use of the product.");
      Put_Line("");
      Put_Line("  ""Installation Information"" for a User Product means any methods,");
      Put_Line("procedures, authorization keys, or other information required to install");
      Put_Line("and execute modified versions of a covered work in that User Product from");
      Put_Line("a modified version of its Corresponding Source.  The information must");
      Put_Line("suffice to ensure that the continued functioning of the modified object");
      Put_Line("code is in no case prevented or interfered with solely because");
      Put_Line("modification has been made.");
      Put_Line("");
      Put_Line("  If you convey an object code work under this section in, or with, or");
      Put_Line("specifically for use in, a User Product, and the conveying occurs as");
      Put_Line("part of a transaction in which the right of possession and use of the");
      Put_Line("User Product is transferred to the recipient in perpetuity or for a");
      Put_Line("fixed term (regardless of how the transaction is characterized), the");
      Put_Line("Corresponding Source conveyed under this section must be accompanied");
      Put_Line("by the Installation Information.  But this requirement does not apply");
      Put_Line("if neither you nor any third party retains the ability to install");
      Put_Line("modified object code on the User Product (for example, the work has");
      Put_Line("been installed in ROM).");
      Put_Line("");
      Put_Line("  The requirement to provide Installation Information does not include a");
      Put_Line("requirement to continue to provide support service, warranty, or updates");
      Put_Line("for a work that has been modified or installed by the recipient, or for");
      Put_Line("the User Product in which it has been modified or installed.  Access to a");
      Put_Line("network may be denied when the modification itself materially and");
      Put_Line("adversely affects the operation of the network or violates the rules and");
      Put_Line("protocols for communication across the network.");
      Put_Line("");
      Put_Line("  Corresponding Source conveyed, and Installation Information provided,");
      Put_Line("in accord with this section must be in a format that is publicly");
      Put_Line("documented (and with an implementation available to the public in");
      Put_Line("source code form), and must require no special password or key for");
      Put_Line("unpacking, reading or copying.");
      Put_Line("");
      Put_Line("  7. Additional Terms.");
      Put_Line("");
      Put_Line("  ""Additional permissions"" are terms that supplement the terms of this");
      Put_Line("License by making exceptions from one or more of its conditions.");
      Put_Line("Additional permissions that are applicable to the entire Program shall");
      Put_Line("be treated as though they were included in this License, to the extent");
      Put_Line("that they are valid under applicable law.  If additional permissions");
      Put_Line("apply only to part of the Program, that part may be used separately");
      Put_Line("under those permissions, but the entire Program remains governed by");
      Put_Line("this License without regard to the additional permissions.");
      Put_Line("");
      Put_Line("  When you convey a copy of a covered work, you may at your option");
      Put_Line("remove any additional permissions from that copy, or from any part of");
      Put_Line("it.  (Additional permissions may be written to require their own");
      Put_Line("removal in certain cases when you modify the work.)  You may place");
      Put_Line("additional permissions on material, added by you to a covered work,");
      Put_Line("for which you have or can give appropriate copyright permission.");
      Put_Line("");
      Put_Line("  Notwithstanding any other provision of this License, for material you");
      Put_Line("add to a covered work, you may (if authorized by the copyright holders of");
      Put_Line("that material) supplement the terms of this License with terms:");
      Put_Line("");
      Put_Line("    a) Disclaiming warranty or limiting liability differently from the");
      Put_Line("    terms of sections 15 and 16 of this License; or");
      Put_Line("");
      Put_Line("    b) Requiring preservation of specified reasonable legal notices or");
      Put_Line("    author attributions in that material or in the Appropriate Legal");
      Put_Line("    Notices displayed by works containing it; or");
      Put_Line("");
      Put_Line("    c) Prohibiting misrepresentation of the origin of that material, or");
      Put_Line("    requiring that modified versions of such material be marked in");
      Put_Line("    reasonable ways as different from the original version; or");
      Put_Line("");
      Put_Line("    d) Limiting the use for publicity purposes of names of licensors or");
      Put_Line("    authors of the material; or");
      Put_Line("");
      Put_Line("    e) Declining to grant rights under trademark law for use of some");
      Put_Line("    trade names, trademarks, or service marks; or");
      Put_Line("");
      Put_Line("    f) Requiring indemnification of licensors and authors of that");
      Put_Line("    material by anyone who conveys the material (or modified versions of");
      Put_Line("    it) with contractual assumptions of liability to the recipient, for");
      Put_Line("    any liability that these contractual assumptions directly impose on");
      Put_Line("    those licensors and authors.");
      Put_Line("");
      Put_Line("  All other non-permissive additional terms are considered ""further");
      Put_Line("restrictions"" within the meaning of section 10.  If the Program as you");
      Put_Line("received it, or any part of it, contains a notice stating that it is");
      Put_Line("governed by this License along with a term that is a further");
      Put_Line("restriction, you may remove that term.  If a license document contains");
      Put_Line("a further restriction but permits relicensing or conveying under this");
      Put_Line("License, you may add to a covered work material governed by the terms");
      Put_Line("of that license document, provided that the further restriction does");
      Put_Line("not survive such relicensing or conveying.");
      Put_Line("");
      Put_Line("  If you add terms to a covered work in accord with this section, you");
      Put_Line("must place, in the relevant source files, a statement of the");
      Put_Line("additional terms that apply to those files, or a notice indicating");
      Put_Line("where to find the applicable terms.");
      Put_Line("");
      Put_Line("  Additional terms, permissive or non-permissive, may be stated in the");
      Put_Line("form of a separately written license, or stated as exceptions;");
      Put_Line("the above requirements apply either way.");
      Put_Line("");
      Put_Line("  8. Termination.");
      Put_Line("");
      Put_Line("  You may not propagate or modify a covered work except as expressly");
      Put_Line("provided under this License.  Any attempt otherwise to propagate or");
      Put_Line("modify it is void, and will automatically terminate your rights under");
      Put_Line("this License (including any patent licenses granted under the third");
      Put_Line("paragraph of section 11).");
      Put_Line("");
      Put_Line("  However, if you cease all violation of this License, then your");
      Put_Line("license from a particular copyright holder is reinstated (a)");
      Put_Line("provisionally, unless and until the copyright holder explicitly and");
      Put_Line("finally terminates your license, and (b) permanently, if the copyright");
      Put_Line("holder fails to notify you of the violation by some reasonable means");
      Put_Line("prior to 60 days after the cessation.");
      Put_Line("");
      Put_Line("  Moreover, your license from a particular copyright holder is");
      Put_Line("reinstated permanently if the copyright holder notifies you of the");
      Put_Line("violation by some reasonable means, this is the first time you have");
      Put_Line("received notice of violation of this License (for any work) from that");
      Put_Line("copyright holder, and you cure the violation prior to 30 days after");
      Put_Line("your receipt of the notice.");
      Put_Line("");
      Put_Line("  Termination of your rights under this section does not terminate the");
      Put_Line("licenses of parties who have received copies or rights from you under");
      Put_Line("this License.  If your rights have been terminated and not permanently");
      Put_Line("reinstated, you do not qualify to receive new licenses for the same");
      Put_Line("material under section 10.");
      Put_Line("");
      Put_Line("  9. Acceptance Not Required for Having Copies.");
      Put_Line("");
      Put_Line("  You are not required to accept this License in order to receive or");
      Put_Line("run a copy of the Program.  Ancillary propagation of a covered work");
      Put_Line("occurring solely as a consequence of using peer-to-peer transmission");
      Put_Line("to receive a copy likewise does not require acceptance.  However,");
      Put_Line("nothing other than this License grants you permission to propagate or");
      Put_Line("modify any covered work.  These actions infringe copyright if you do");
      Put_Line("not accept this License.  Therefore, by modifying or propagating a");
      Put_Line("covered work, you indicate your acceptance of this License to do so.");
      Put_Line("");
      Put_Line("  10. Automatic Licensing of Downstream Recipients.");
      Put_Line("");
      Put_Line("  Each time you convey a covered work, the recipient automatically");
      Put_Line("receives a license from the original licensors, to run, modify and");
      Put_Line("propagate that work, subject to this License.  You are not responsible");
      Put_Line("for enforcing compliance by third parties with this License.");
      Put_Line("");
      Put_Line("  An ""entity transaction"" is a transaction transferring control of an");
      Put_Line("organization, or substantially all assets of one, or subdividing an");
      Put_Line("organization, or merging organizations.  If propagation of a covered");
      Put_Line("work results from an entity transaction, each party to that");
      Put_Line("transaction who receives a copy of the work also receives whatever");
      Put_Line("licenses to the work the party's predecessor in interest had or could");
      Put_Line("give under the previous paragraph, plus a right to possession of the");
      Put_Line("Corresponding Source of the work from the predecessor in interest, if");
      Put_Line("the predecessor has it or can get it with reasonable efforts.");
      Put_Line("");
      Put_Line("  You may not impose any further restrictions on the exercise of the");
      Put_Line("rights granted or affirmed under this License.  For example, you may");
      Put_Line("not impose a license fee, royalty, or other charge for exercise of");
      Put_Line("rights granted under this License, and you may not initiate litigation");
      Put_Line("(including a cross-claim or counterclaim in a lawsuit) alleging that");
      Put_Line("any patent claim is infringed by making, using, selling, offering for");
      Put_Line("sale, or importing the Program or any portion of it.");
      Put_Line("");
      Put_Line("  11. Patents.");
      Put_Line("");
      Put_Line("  A ""contributor"" is a copyright holder who authorizes use under this");
      Put_Line("License of the Program or a work on which the Program is based.  The");
      Put_Line("work thus licensed is called the contributor's ""contributor version"".");
      Put_Line("");
      Put_Line("  A contributor's ""essential patent claims"" are all patent claims");
      Put_Line("owned or controlled by the contributor, whether already acquired or");
      Put_Line("hereafter acquired, that would be infringed by some manner, permitted");
      Put_Line("by this License, of making, using, or selling its contributor version,");
      Put_Line("but do not include claims that would be infringed only as a");
      Put_Line("consequence of further modification of the contributor version.  For");
      Put_Line("purposes of this definition, ""control"" includes the right to grant");
      Put_Line("patent sublicenses in a manner consistent with the requirements of");
      Put_Line("this License.");
      Put_Line("");
      Put_Line("  Each contributor grants you a non-exclusive, worldwide, royalty-free");
      Put_Line("patent license under the contributor's essential patent claims, to");
      Put_Line("make, use, sell, offer for sale, import and otherwise run, modify and");
      Put_Line("propagate the contents of its contributor version.");
      Put_Line("");
      Put_Line("  In the following three paragraphs, a ""patent license"" is any express");
      Put_Line("agreement or commitment, however denominated, not to enforce a patent");
      Put_Line("(such as an express permission to practice a patent or covenant not to");
      Put_Line("sue for patent infringement).  To ""grant"" such a patent license to a");
      Put_Line("party means to make such an agreement or commitment not to enforce a");
      Put_Line("patent against the party.");
      Put_Line("");
      Put_Line("  If you convey a covered work, knowingly relying on a patent license,");
      Put_Line("and the Corresponding Source of the work is not available for anyone");
      Put_Line("to copy, free of charge and under the terms of this License, through a");
      Put_Line("publicly available network server or other readily accessible means,");
      Put_Line("then you must either (1) cause the Corresponding Source to be so");
      Put_Line("available, or (2) arrange to deprive yourself of the benefit of the");
      Put_Line("patent license for this particular work, or (3) arrange, in a manner");
      Put_Line("consistent with the requirements of this License, to extend the patent");
      Put_Line("license to downstream recipients.  ""Knowingly relying"" means you have");
      Put_Line("actual knowledge that, but for the patent license, your conveying the");
      Put_Line("covered work in a country, or your recipient's use of the covered work");
      Put_Line("in a country, would infringe one or more identifiable patents in that");
      Put_Line("country that you have reason to believe are valid.");
      Put_Line("");
      Put_Line("  If, pursuant to or in connection with a single transaction or");
      Put_Line("arrangement, you convey, or propagate by procuring conveyance of, a");
      Put_Line("covered work, and grant a patent license to some of the parties");
      Put_Line("receiving the covered work authorizing them to use, propagate, modify");
      Put_Line("or convey a specific copy of the covered work, then the patent license");
      Put_Line("you grant is automatically extended to all recipients of the covered");
      Put_Line("work and works based on it.");
      Put_Line("");
      Put_Line("  A patent license is ""discriminatory"" if it does not include within");
      Put_Line("the scope of its coverage, prohibits the exercise of, or is");
      Put_Line("conditioned on the non-exercise of one or more of the rights that are");
      Put_Line("specifically granted under this License.  You may not convey a covered");
      Put_Line("work if you are a party to an arrangement with a third party that is");
      Put_Line("in the business of distributing software, under which you make payment");
      Put_Line("to the third party based on the extent of your activity of conveying");
      Put_Line("the work, and under which the third party grants, to any of the");
      Put_Line("parties who would receive the covered work from you, a discriminatory");
      Put_Line("patent license (a) in connection with copies of the covered work");
      Put_Line("conveyed by you (or copies made from those copies), or (b) primarily");
      Put_Line("for and in connection with specific products or compilations that");
      Put_Line("contain the covered work, unless you entered into that arrangement,");
      Put_Line("or that patent license was granted, prior to 28 March 2007.");
      Put_Line("");
      Put_Line("  Nothing in this License shall be construed as excluding or limiting");
      Put_Line("any implied license or other defenses to infringement that may");
      Put_Line("otherwise be available to you under applicable patent law.");
      Put_Line("");
      Put_Line("  12. No Surrender of Others' Freedom.");
      Put_Line("");
      Put_Line("  If conditions are imposed on you (whether by court order, agreement or");
      Put_Line("otherwise) that contradict the conditions of this License, they do not");
      Put_Line("excuse you from the conditions of this License.  If you cannot convey a");
      Put_Line("covered work so as to satisfy simultaneously your obligations under this");
      Put_Line("License and any other pertinent obligations, then as a consequence you may");
      Put_Line("not convey it at all.  For example, if you agree to terms that obligate you");
      Put_Line("to collect a royalty for further conveying from those to whom you convey");
      Put_Line("the Program, the only way you could satisfy both those terms and this");
      Put_Line("License would be to refrain entirely from conveying the Program.");
      Put_Line("");
      Put_Line("  13. Use with the GNU Affero General Public License.");
      Put_Line("");
      Put_Line("  Notwithstanding any other provision of this License, you have");
      Put_Line("permission to link or combine any covered work with a work licensed");
      Put_Line("under version 3 of the GNU Affero General Public License into a single");
      Put_Line("combined work, and to convey the resulting work.  The terms of this");
      Put_Line("License will continue to apply to the part which is the covered work,");
      Put_Line("but the special requirements of the GNU Affero General Public License,");
      Put_Line("section 13, concerning interaction through a network will apply to the");
      Put_Line("combination as such.");
      Put_Line("");
      Put_Line("  14. Revised Versions of this License.");
      Put_Line("");
      Put_Line("  The Free Software Foundation may publish revised and/or new versions of");
      Put_Line("the GNU General Public License from time to time.  Such new versions will");
      Put_Line("be similar in spirit to the present version, but may differ in detail to");
      Put_Line("address new problems or concerns.");
      Put_Line("");
      Put_Line("  Each version is given a distinguishing version number.  If the");
      Put_Line("Program specifies that a certain numbered version of the GNU General");
      Put_Line("Public License ""or any later version"" applies to it, you have the");
      Put_Line("option of following the terms and conditions either of that numbered");
      Put_Line("version or of any later version published by the Free Software");
      Put_Line("Foundation.  If the Program does not specify a version number of the");
      Put_Line("GNU General Public License, you may choose any version ever published");
      Put_Line("by the Free Software Foundation.");
      Put_Line("");
      Put_Line("  If the Program specifies that a proxy can decide which future");
      Put_Line("versions of the GNU General Public License can be used, that proxy's");
      Put_Line("public statement of acceptance of a version permanently authorizes you");
      Put_Line("to choose that version for the Program.");
      Put_Line("");
      Put_Line("  Later license versions may give you additional or different");
      Put_Line("permissions.  However, no additional obligations are imposed on any");
      Put_Line("author or copyright holder as a result of your choosing to follow a");
      Put_Line("later version.");
      Put_Line("");
      Put_Line("  15. Disclaimer of Warranty.");
      Put_Line("");
      Put_Line("  THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY");
      Put_Line("APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT");
      Put_Line("HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM ""AS IS"" WITHOUT WARRANTY");
      Put_Line("OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,");
      Put_Line("THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR");
      Put_Line("PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM");
      Put_Line("IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF");
      Put_Line("ALL NECESSARY SERVICING, REPAIR OR CORRECTION.");
      Put_Line("");
      Put_Line("  16. Limitation of Liability.");
      Put_Line("");
      Put_Line("  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING");
      Put_Line("WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS");
      Put_Line("THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY");
      Put_Line("GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE");
      Put_Line("USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF");
      Put_Line("DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD");
      Put_Line("PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),");
      Put_Line("EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF");
      Put_Line("SUCH DAMAGES.");
      Put_Line("");
      Put_Line("  17. Interpretation of Sections 15 and 16.");
      Put_Line("");
      Put_Line("  If the disclaimer of warranty and limitation of liability provided");
      Put_Line("above cannot be given local legal effect according to their terms,");
      Put_Line("reviewing courts shall apply local law that most closely approximates");
      Put_Line("an absolute waiver of all civil liability in connection with the");
      Put_Line("Program, unless a warranty or assumption of liability accompanies a");
      Put_Line("copy of the Program in return for a fee.");
      Put_Line("");
      Put_Line("                     END OF TERMS AND CONDITIONS");
      Put_Line("");
      Put_Line("            How to Apply These Terms to Your New Programs");
      Put_Line("");
      Put_Line("  If you develop a new program, and you want it to be of the greatest");
      Put_Line("possible use to the public, the best way to achieve this is to make it");
      Put_Line("free software which everyone can redistribute and change under these terms.");
      Put_Line("");
      Put_Line("  To do so, attach the following notices to the program.  It is safest");
      Put_Line("to attach them to the start of each source file to most effectively");
      Put_Line("state the exclusion of warranty; and each file should have at least");
      Put_Line("the ""copyright"" line and a pointer to where the full notice is found.");
      Put_Line("");
      Put_Line("    <one line to give the program's name and a brief idea of what it does.>");
      Put_Line("    Copyright (C) <year>  <name of author>");
      Put_Line("");
      Put_Line("    This program is free software: you can redistribute it and/or modify");
      Put_Line("    it under the terms of the GNU General Public License as published by");
      Put_Line("    the Free Software Foundation, either version 3 of the License, or");
      Put_Line("    (at your option) any later version.");
      Put_Line("");
      Put_Line("    This program is distributed in the hope that it will be useful,");
      Put_Line("    but WITHOUT ANY WARRANTY; without even the implied warranty of");
      Put_Line("    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the");
      Put_Line("    GNU General Public License for more details.");
      Put_Line("");
      Put_Line("    You should have received a copy of the GNU General Public License");
      Put_Line("    along with this program.  If not, see <http://www.gnu.org/licenses/>.");
      Put_Line("");
      Put_Line("Also add information on how to contact you by electronic and paper mail.");
      Put_Line("");
      Put_Line("  If the program does terminal interaction, make it output a short");
      Put_Line("notice like this when it starts in an interactive mode:");
      Put_Line("");
      Put_Line("    <program>  Copyright (C) <year>  <name of author>");
      Put_Line("    This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.");
      Put_Line("    This is free software, and you are welcome to redistribute it");
      Put_Line("    under certain conditions; type `show c' for details.");
      Put_Line("");
      Put_Line("The hypothetical commands `show w' and `show c' should show the appropriate");
      Put_Line("parts of the General Public License.  Of course, your program's commands");
      Put_Line("might be different; for a GUI interface, you would use an ""about box"".");
      Put_Line("");
      Put_Line("  You should also get your employer (if you work as a programmer) or school,");
      Put_Line("if any, to sign a ""copyright disclaimer"" for the program, if necessary.");
      Put_Line("For more information on this, and how to apply and follow the GNU GPL, see");
      Put_Line("<http://www.gnu.org/licenses/>.");
      Put_Line("");
      Put_Line("  The GNU General Public License does not permit incorporating your program");
      Put_Line("into proprietary programs.  If your program is a subroutine library, you");
      Put_Line("may consider it more useful to permit linking proprietary applications with");
      Put_Line("the library.  If this is what you want to do, use the GNU Lesser General");
      Put_Line("Public License instead of this License.  But first, please read");
      Put_Line("<http://www.gnu.org/philosophy/why-not-lgpl.html>.");
   end Show_GPL;

begin

   declare
      I: Integer := 1;
   begin

      while I <= Argument_Count loop

         if Argument(I) = "-v" then
            Options.Verbosity := 1;

         elsif Argument(I) = "-vv" then
            Options.Verbosity := 2;

         elsif Argument(I) = "-vvv" then
            Options.Verbosity := 3;

         elsif Argument(I) = "-vvvv" then
            Options.Verbosity := 4;

         elsif Argument(I) = "-i" then
            Options.Indirect_Candidates := True;
         elsif Argument(I) = "-t" then
            Options.Light_Scan := True;
         elsif Argument(I) = "-l" then
            Show_GPL;
            Set_Exit_Status(Success);
            return;

         elsif Argument(I) = "-f" then
            I := I + 1;

            if I = Argument_Count then
               if Argument(I)(1) = '-' then
                  Set_Exit_Status(Failure);
                  Put_Line(Standard_Error, "[BWAA] no option allowed after -f");
                  Put_Line(Standard_Error, "");
                  Usage;
                  return;
               end if;
               Options.Context_Parameters := To_Unbounded_String("-C1 " & Argument(I));
            elsif I < Argument_Count then
               Options.Context_Parameters := To_Unbounded_String("-CN");
               while I <= Argument_Count loop
                  if Argument(I)(1) = '-' then
                     Set_Exit_Status(Failure);
                     Put_Line(Standard_Error, "[BWAA] no option allowed after -f");
                     Put_Line(Standard_Error, "");
                     Usage;
                     return;
                  end if;
                  Options.Context_Parameters := Options.Context_Parameters & To_Unbounded_String(" " & Argument(I));
                  I := I + 1;
               end loop;
            else
               Set_Exit_Status(Failure);
               Put_Line(Standard_Error, "[BWAA] -f given, but no files");
               Put_Line(Standard_Error, "");
               Usage;
               return;
            end if;
         else
            Set_Exit_Status(Failure);
            Put_Line(Standard_Error, "[BWAA] Unknown option: " & Argument(I));
            Put_Line(Standard_Error, "");
            Usage;
            return;
         end if;

         I := I + 1;
      end loop;
   end;

   Main.Run;
end Run_Bwaa;
